/**
 * @file OsalDevDrvCommon.h
 *
 * @brief Common device driver and macro types
 *
 *
 * @par
 *   SPDX-License-Identifier: BSD-3-Clause
 *   Copyright(c) 2007-2026 Intel Corporation
 * 
 *   These contents may have been developed with support from one or more
 *   Intel-operated generative artificial intelligence solutions.
 *
 */

#ifndef OSAL_DEV_DRV_COMMON_H
#define OSAL_DEV_DRV_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif
#endif
